/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include "fileio.h"
#include "syntax.h"
#include "utils.h"

// Create a new project
void openldev::on_file_new()
{  
  new_project_dialog *dialog;
  dialog = new new_project_dialog();
  int result = dialog->run();

  // Open newly created project if OK is pressed
  switch (result)
  {
  case Gtk::RESPONSE_ACCEPT:
    open_project(dialog->get_filename());
    break;
  }
  
  dialog->hide();
}

// Create a new file
void openldev::on_file_new_file()
{  
  if (proj.get_name().empty())
  {
    error("Please open or create\na project first!");
  } 
  // Only create new file if project is open
  else 
  {
    new_file_dialog *dialog;
    dialog = new new_file_dialog (proj);
    int result = dialog->run();

    // Open the newly created file
    switch (result)
    {
    case Gtk::RESPONSE_ACCEPT:
      open_file(dialog->get_filename());
      break;
    }
  
    dialog->hide();
  }
}

// Open a file or project
void openldev::on_file_open()
{
  // Create file selector
  Gtk::FileChooserDialog dialog("Choose a Project or File", Gtk::FILE_CHOOSER_ACTION_OPEN);
  dialog.set_transient_for(*this);
  dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
  dialog.add_button(Gtk::Stock::OPEN, Gtk::RESPONSE_OK);
  
  // Set default folder
  if (!proj.get_name().empty())
    dialog.set_current_folder(proj.get_location());
  else
    dialog.set_current_folder(Glib::get_home_dir());
  
  int result = dialog.run();
  switch (result)
  {
    // If OK is pressed
    case (Gtk::RESPONSE_OK):
    {
      // Check if file is a .odp or .openldev file
      Glib::ustring filename = dialog.get_filename();
      if (filename.find(".odp", 0) != string::npos || 
          filename.find(".openldev", 0) != string::npos)
        open_project(filename);
      else
        open_file(filename);    
      break;
    }
    default:
      break;
  }
}

// Open File
void openldev::open_file(Glib::ustring filename)
{
  Glib::FileTest filetest = Glib::FILE_TEST_EXISTS;
  if (!Glib::file_test(filename, filetest))
  {
    error("Cannot open file! It does not exist!");
  }
  else
  {
    // Add a new tab to the notebookmanager
    if (noteman->add_tab(filename, env))
    { 
      Glib::ustring text, title, cliptext;
    
      Glib::RefPtr<Gtk::Clipboard> Clipboard = Gtk::Clipboard::get();
      Clipboard->set_text(screen.get_clipboard());
  
      // Change last file open in project
      if (!proj.get_name().empty()) 
      {
        Glib::ustring fn = filename;
        unsigned int location = fn.find('/', 0);
        while (location != Glib::ustring::npos) 
        {
          fn.erase(0,location+1);
          location = fn.find('/', 0);
        }
        if (proj.get_source_files().find(fn, 0) != Glib::ustring::npos ||
            proj.get_header_files().find(fn, 0) != Glib::ustring::npos) 
        {
          proj.set_lastfile(fn);
          proj.write_project_file();
        }
      }
  
      // Open file & read in text
      screen.set_filename(filename);
      text = read_file(noteman->get_current_filename());
      
      // Add text to text buffer after applying syntax highlighting
      Glib::RefPtr<Gtk::TextBuffer> text_buffer = Gtk::TextBuffer::create();
      text_buffer->set_text(text);
      set_syntax(text_buffer, true, true, env);
      
      // Set the text to the buffer, connect keystroke function & show/hide line numbering
      noteman->get_current_textview()->set_buffer(text_buffer);
      noteman->get_current_textview()->signal_key_press_event().connect(sigc::mem_fun(*this, 
          &openldev::on_keystroke), false);
      noteman->get_current_textview()->get_buffer()->signal_mark_set().connect(sigc::mem_fun(*this, 
          &openldev::update_status_bar), false);
      noteman->get_aman().erase();
      noteman->create_line_numbers();
      noteman->scroll_to_mark();
      
      if (!env.get_line_numbers())
        noteman->show_line_numbers(false);
    }
  }
}

// Open project
void openldev::open_project(Glib::ustring filename)
{
  noteman->erase_all_tabs();
  bool success = proj.load_project_file(filename);
 
  // If the project was opened
  if (success)
  {
    // Open the last file if it exists
    if (!proj.get_lastfile().empty())
    {
      Glib::ustring fn = Glib::ustring(proj.get_location()) + "src/" 
        + Glib::ustring(proj.get_lastfile());
      open_file(fn);
    }

    // Make environment ready for project
    populate_treeview();
	  env.set_lastproject(filename);
    env.write_settings_file();
  }
}

// Save current file or call save as if there is no file open
void openldev::on_file_save()
{
  if (noteman->get_current_filename().empty() || noteman->get_current_filename() == "EMPTY")
    on_file_saveas();
  
  // Save the file & set as unmodified
  Glib::RefPtr<Gtk::TextBuffer> text_buffer = Gtk::TextBuffer::create();
  text_buffer = noteman->get_current_textview()->get_buffer();
  write_file(noteman->get_current_filename(), text_buffer->get_text(true));
  noteman->mark_current_tab_modified(false);
}

// Save current file by a different file name
void openldev::on_file_saveas() 
{
  Gtk::FileChooserDialog dialog("Save File as", Gtk::FILE_CHOOSER_ACTION_SAVE);
  dialog.set_transient_for(*this);
  dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
  dialog.add_button(Gtk::Stock::SAVE, Gtk::RESPONSE_OK);
  
  // Set default folder
  if (!proj.get_name().empty())
    dialog.set_current_folder(Glib::ustring(proj.get_location() + "src/"));
  else
    dialog.set_current_folder(Glib::get_home_dir());
  
  // Add the filename to the text entry in SaveAs dialog
  if (noteman->get_current_filename().empty() || noteman->get_current_filename() == "EMPTY")
    dialog.set_current_name("untitled");
  else
  {
    Glib::ustring fn = noteman->get_current_filename();
    while (fn.find("/", 0) != string::npos)
      fn.erase(0, fn.find("/", 0) + 1);
    
    dialog.set_current_name(fn);
  }
  
  // Open fileselection dialog to get file location
  int result = dialog.run();
  switch (result)
  {
    // If OK is pressed
    case(Gtk::RESPONSE_OK):
      screen.set_filename(dialog.get_filename());  
      on_file_save();
      break;
    default:
      break;
  }
}

// Save all currently open files
void openldev::on_file_saveall()
{
  noteman->save_all_files();
}
