/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "notebookmanager.h"
#include "utils.h"
#include "syntax.h"

// Default notebook constructor
notebook_manager::notebook_manager()
{
  // Create the notebook & set its settings
	notebook = new Gtk::Notebook();
	notebook->set_show_tabs(true);
  notebook->set_tab_pos(Gtk::POS_TOP);
  notebook->set_scrollable(true);
	notebook->show();
	tab_count = 0;
}

// Get the notebook widget
Gtk::Notebook* notebook_manager::get_notebook()
{
	return notebook;
}

// Add a tab to the notebook
bool notebook_manager::add_tab(Glib::ustring filename, environment_settings env)
{
  notebook_tab tab;
  tab.fn = filename;
  
  // Parse the filename to remove directory structure
  while (filename.find('/',0) != string::npos)
    filename.erase(0,filename.find('/',0)+1);

  // Check to make sure file is not already open
  for (int i=0; i<tab_count; i++)
  {
    Glib::ustring temp = tabs[i].label->get_text();
    if (temp.find('*', 0) != string::npos)
      temp.erase(temp.find('*', 0), 1);
    
    if (filename == temp)
    {
      notebook->set_current_page(i);
      return false;
    }
  }

  // Create temporary tab & initialize the objects
	tab.initialize_objects(filename);
	tab.tab_id = tab_count;

  // Add the tab to the notebook and notebook_tab's deque
	tabs.push_back(tab);
	notebook->append_page(*tabs[tab_count].hbox2, *tabs[tab_count].hbox);
  
  // Set all of the signal commands
	tabs[tab_count].button->signal_clicked().connect(sigc::bind<Glib::ustring>(sigc::mem_fun(*this, 
          &notebook_manager::delete_tab), tabs[tab_count].fn));
  tabs[tab_count].scrolled_win->get_vadjustment()->signal_changed().connect(sigc::mem_fun(*this,
          &notebook_manager::sync_scrollbars));
   
  // Set the font and tab size of the textview  
  Glib::ustring font_description = env.get_font_face() + " " + env.get_font_size();
  Pango::FontDescription fd(font_description);  
  tabs[tab_count].textview->modify_font(fd);
  tabs[tab_count].line_numbers->modify_font(fd);
  tabs[tab_count].textview->set_tabs(get_tab_array(fd, env, tabs[tab_count].textview));
  
  // Do final initialization
  notebook->set_current_page(tab_count);
	tab_count++;
  return true;
}

// Delete the currently selected tab
void notebook_manager::delete_tab(Glib::ustring fn)
{
  // Find file button that was clicked & get buffer
  int tab = 0;
  while (fn != tabs[tab].fn)
    tab++;
  Glib::RefPtr<Gtk::TextBuffer> text_buffer = tabs[tab].textview->get_buffer();
  
  // Save the file before closing
  write_file(tabs[tab].fn, text_buffer->get_text(true));
  
  // Remove the tab
	deque<notebook_tab>::iterator itr = tabs.begin();
	notebook->remove_page(tab);
	
  // Move the iter to the current tab
	for (int i=0; i < tab; i++)
		itr++;
	
  // Remove the tab objects & information
  tab_count--;
	tabs.erase(itr);
  
  // Set the current page to the previous one if there is a tab open
  if (tab_count != 0)
    notebook->set_current_page(tab_count);
}

// Save all files in the tabs
void notebook_manager::save_all_files()
{
  for (int i=0; i<notebook->get_n_pages(); i++)
  {
    // Read in the current text of the file
    Glib::ustring temporary_text = read_file(tabs[i].fn);
    
    // Set the label as unmodified
    Glib::ustring text = tabs[i].label->get_text();
    if (text.find('*', 0) != string::npos)
      text.erase(text.find('*', 0), 1);
    tabs[i].label->set_text(text);
    
    // Check to see if the file has been modified, & save the file if so
    if (temporary_text != tabs[i].textview->get_buffer()->get_text(true))
    {
      write_file(tabs[i].fn, tabs[i].textview->get_buffer()->get_text(true));
    }
  }
}

// Destroy all information
void notebook_manager::erase_all_tabs()
{
  // Remove all of the tabs and tab information
  tabs.erase(tabs.begin(), tabs.end());
  while (notebook->get_n_pages() != 0)
    notebook->remove_page(0);
  tab_count = 0;
}

// Get the textview for editing
Gtk::TextView* notebook_manager::get_current_textview()
{
  int current = notebook->get_current_page();

  // Return the current textview
  if (tab_count != 0)
    return tabs[current].textview;
  // Return an empty textview if there are none
  else
  {
    Gtk::TextView *textview = new Gtk::TextView();
    return textview;
  }
}

// Return the current textview's Action Manager
action_manager& notebook_manager::get_aman()
{
  return tabs[notebook->get_current_page()].aman;
}

// Return the current tab
int notebook_manager::get_current_tab()
{
  return notebook->get_current_page();
}

// Mark the file modified or unmodified
void notebook_manager::mark_current_tab_modified(bool modified)
{
  int current = notebook->get_current_page();
  Glib::ustring text = tabs[current].label->get_text();
  
  // If the file was modified
  if (modified)
  {
    if (text.find('*', 0) == string::npos)
      text += "*";
  }
  // If it was not modified
  else 
  {
    if (text.find('*', 0) != string::npos)
      text.erase(text.find('*', 0), 1);
  }
  
  tabs[current].label->set_text(text);
}


// Modify the font of each textview & line numbering
void notebook_manager::modify_fonts(Pango::FontDescription fd)
{
  // Foreach tab, modify the fonts
  for (int i=0; i<tab_count; i++)
  {
    tabs[i].textview->modify_font(fd);
    tabs[i].line_numbers->modify_font(fd);
  }
}

// Scroll to the current cursor position
void notebook_manager::scroll_to_mark()
{
  int current = notebook->get_current_page();
  tabs[current].textview->scroll_to(tabs[current].textview->get_buffer()->get_insert(), 0.0);
}

// Get the filename of the current tab
Glib::ustring notebook_manager::get_current_filename()
{
  int current = notebook->get_current_page();
  return tabs[current].fn;
}

// Get the short filename of the current tab
Glib::ustring notebook_manager::get_current_short_filename()
{
  int current = notebook->get_current_page();
  Glib::ustring fn = tabs[current].label->get_text();
  if (fn.find("*", 0) != string::npos)
    fn.erase(fn.find("*", 0), 1);
  return fn;
}

// Create the line numbering for a file
void notebook_manager::create_line_numbers()
{
  int current = notebook->get_current_page();
  int lines = tabs[current].textview->get_buffer()->get_line_count()+1;
  Glib::ustring text;
  
  // Loop through the lines & add each line number to the string
  for (int i=1; i<lines; i++)
  {
    gchar number[4];
    sprintf(number, "%d", i);
    text += Glib::ustring(number) + "\n";
  }
  
  // Erase the last endl and then set the line numbers text
  text.erase(text.length()-1, 1);
  tabs[current].line_numbers->set_text(text);
  tabs[current].current_line_count = lines - 1;
}

// Update the line numbering
void notebook_manager::update_line_numbers()
{
  int current = notebook->get_current_page();
  int lines = tabs[current].textview->get_buffer()->get_line_count();
  
  // Compare the true number of lines and the shown number of lines
  if (tabs[current].current_line_count != lines)
    create_line_numbers();
}

void notebook_manager::show_line_numbers(bool show_or_hide)
{
  for (int i=0; i<tab_count; i++)
  {
    if (show_or_hide)
    {
      tabs[i].line_numbers->show();
      tabs[i].swin->show();
    }
    else
    {
      tabs[i].line_numbers->hide();
      tabs[i].swin->hide();
    }
  }
}

void notebook_manager::sync_scrollbars()
{
  int current = notebook->get_current_page();
  tabs[notebook->get_current_page()].swin->set_vadjustment(tabs[current].scrolled_win->get_vadjustment());
}

notebook_manager::~notebook_manager()
{
}
